"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lang_1 = require("./lang");
const map_1 = require("../util/map");
const exception_1 = require("./exception");
class StateMachine {
    constructor(factory) {
        this.currentStatus = null;
        this.isStartup = false;
        this.isFinish = false;
        this.nextStatus = null;
        this.result = null;
        this.transitionMapping = factory.transitionMapping;
        this.stateRules = factory.stateRules;
        this.catcher = factory.catcher;
        this.currentStatus = factory.initialStatus;
        let that = this;
        this.resolve = new class {
            status(status) {
                if (status in that.stateRules && that.stateRules[that.currentStatus].containsKey(status)) {
                    that.nextStatus = status;
                    return this;
                }
                throw new exception_1.IllegalStateException();
            }
            resolve(result) {
                that.result = result;
                return this;
            }
        };
        factory = null;
    }
    start() {
        if (this.isStartup) {
            throw new exception_1.IllegalStateException("");
        }
        let mapping = this.transitionMapping[null];
        if (!lang_1.CoreUtils.isNone(mapping) && mapping.containsKey(this.currentStatus)) {
            this.result = mapping.get(this.currentStatus)();
        }
        this.isStartup = true;
    }
    sendEvent(event) {
        if (!this.isStartup) {
            return Promise.reject(new exception_1.IllegalStateException("State machine is not start."));
        }
        if (this.isFinish) {
            return Promise.reject(new exception_1.IllegalStateException("State machine is closed."));
        }
        if (!(event in this.stateRules)) {
            return Promise.reject(new exception_1.IllegalStateException(`Event '${event}' is not found, maybe you forget to add this event.`));
        }
        let rules = this.stateRules[event];
        if (!rules.containsKey(this.currentStatus)) {
            return Promise.reject(new exception_1.IllegalStateException(``));
        }
        return this.transitionTo(rules.get(this.currentStatus));
    }
    transitionTo(target) {
        if (!this.isStartup) {
            return Promise.reject(new exception_1.IllegalStateException("State machine is not start."));
        }
        if (this.isFinish) {
            return Promise.reject(new exception_1.IllegalStateException("State machine is closed."));
        }
        // clear
        this.nextStatus = null;
        try {
            let mapping = this.transitionMapping[this.currentStatus];
            if (mapping.containsKey(target)) {
                this.result = mapping.get(target)(this.result);
            }
            this.currentStatus = target;
            mapping = this.transitionMapping[target];
            this.isFinish = lang_1.CoreUtils.isNone(mapping) || mapping.isEmpty();
        }
        catch (e) {
            let mapping = this.catcher[this.currentStatus];
            if (!mapping.containsKey(target)) {
                return Promise.reject(new exception_1.IllegalStateException(e));
            }
            else {
                try {
                    mapping.get(target)(e, this.resolve);
                    // maybe dead loop
                    if (!lang_1.CoreUtils.isNone(this.nextStatus)) {
                        this.currentStatus = this.nextStatus;
                    }
                    mapping = this.transitionMapping[this.nextStatus];
                    this.isFinish = lang_1.CoreUtils.isNone(mapping) || mapping.isEmpty();
                }
                catch (e) {
                    return Promise.reject(e);
                }
            }
        }
        return Promise.resolve(this.result);
    }
    isDone() {
        return this.isFinish;
    }
}
exports.StateMachine = StateMachine;
class StateMachineFactory {
    constructor() {
        this.initialStatus = "";
        this.events = new map_1.HashMap();
        this.stateRules = Object.create(null);
        this.catcher = Object.create(null);
        this.transitionMapping = Object.create(null);
    }
    addRule(source, target, event) {
        if (!(event in this.stateRules)) {
            this.stateRules[event] = new map_1.HashMap();
        }
        let rules = this.stateRules[event];
        if (lang_1.CoreUtils.isString(source) || null === source) {
            source = [source];
        }
        if (lang_1.CoreUtils.isString(target) || null === source) {
            target = [target];
        }
        source.forEach(s => target.forEach(t => rules.put(s, t)));
        return this;
    }
    addAction(source, target, processor) {
        if (lang_1.CoreUtils.isString(source) || null === source) {
            source = [source];
        }
        if (lang_1.CoreUtils.isString(target) || null === source) {
            target = [target];
        }
        source.forEach(s => {
            if (!(s in this.transitionMapping)) {
                this.transitionMapping[s] = new map_1.HashMap();
            }
            let mapping = this.transitionMapping[s];
            target.forEach(t => mapping.put(t, processor));
        });
        return this;
    }
    addExceptionCatcher(source, target, catcher) {
        if (lang_1.CoreUtils.isString(source) || null === source) {
            source = [source];
        }
        if (lang_1.CoreUtils.isString(target) || null === source) {
            target = [target];
        }
        source.forEach(s => {
            if (!(s in this.catcher)) {
                this.catcher[s] = new map_1.HashMap();
            }
            let mapping = this.catcher[s];
            target.forEach(t => mapping.put(t, catcher));
        });
        return this;
    }
    addStateDetails(source, target, event, processor, catcher) {
        this.addRule(source, target, event);
        if (processor instanceof Function) {
            this.addAction(source, target, processor);
        }
        if (catcher instanceof Function) {
            this.addExceptionCatcher(source, target, catcher);
        }
        return this;
    }
    setInitialStatus(status) {
        this.initialStatus = status;
        return this;
    }
    build() {
        return new StateMachine({
            initialStatus: this.initialStatus,
            events: this.events,
            stateRules: this.stateRules,
            catcher: this.catcher,
            transitionMapping: this.transitionMapping
        });
    }
}
exports.StateMachineFactory = StateMachineFactory;
//# sourceMappingURL=stateMachine.js.map